import { EnchantmentType, EntityComponentTypes, EquipmentSlot, ItemStack, system, world } from "@minecraft/server";
import { MessageFormData, ModalFormData } from "@minecraft/server-ui"
import { VFX_particle, fetchDynamicData, randomNumber, updateDynamicData, writeDynamicData, RedstonePowerChecker } from "../extended_functions.js";

const dynamicFormat = (block) => `${block.typeId.split(":")[1]}=${JSON.stringify(block.location)}`

world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent('ext:auto_harvest_component', {
        onTick: autoHarvest_onTick,
        onPlayerInteract: autoHarvest_interact
    });
});

// Ao usar bonemeal enquanto estiver agachado, adiciona a quantidade ao armazenamento do bloco.
world.beforeEvents.itemUseOn.subscribe((event) => {
    const { source, itemStack, block, isFirstEvent } = event;
    const dynamicName = dynamicFormat(block);
    const dynamicData = fetchDynamicData(dynamicName) || {};
    if (!isFirstEvent) return;
    if (source.isSneaking && itemStack.typeId === "minecraft:bone_meal") {
        source.runCommandAsync(`replaceitem entity @s slot.weapon.mainhand 0 air`)
        const newBoneMeal = (dynamicData.boneMeal_storage || 0) + itemStack.amount;
        writeDynamicData(dynamicName, {
            ...dynamicData,
            boneMeal_storage: newBoneMeal,
            time: autoHarvest_config[block.typeId].time
        });
    }
});

function autoHarvest_interact(event) {
    const { player, block, dimension } = event;
    const dynamicName = dynamicFormat(block);
    const dynamicData = fetchDynamicData(dynamicName) || {};

    function interact_mainGUI() {
        new MessageFormData()
            .title({ translate: "tile.ext:auto_harvest_gui.text.title" })
            .body({ rawtext: [{ text: " \n§e" }, { translate: "tile.ext:auto_harvest_gui.text.1" }, { text: ` §a${dynamicData.boneMeal_storage}` }] })
            .button2({ translate: "tile.ext:auto_harvest_gui.text.button1" })
            .button1({ translate: "tile.ext:auto_harvest_gui.text.button2" })
            .show(player).then((response) => {
                if (response.selection == 1) {
                    interact_removeFertilizer()
                }
            })
    }

    function interact_removeFertilizer() {
        new ModalFormData()
            .title({ translate: "tile.ext:auto_harvest_gui.text.title" })
            .textField({ rawtext: [{ translate: "tile.ext:auto_harvest_gui.text.2" }] }, `${Math.round(dynamicData.boneMeal_storage / 2)}`)
            .submitButton({ translate: "tile.ext:auto_harvest_gui.text.button3" })
            .show(player).then((response) => {
                const { formValues, canceled } = response
                if (canceled) return;
                const valueRemove = parseInt(formValues[0])
                const slotUseCalculate = Math.ceil(valueRemove / 64)
                const inventoryComponent = player.getComponent("minecraft:inventory")
                if (valueRemove <= dynamicData.boneMeal_storage) {
                    if (inventoryComponent.container.emptySlotsCount >= slotUseCalculate) {
                        for (let index = 0; index < valueRemove; index++) {
                            inventoryComponent.container.addItem(new ItemStack("minecraft:bone_meal", 1))
                        }
                    } else {
                        dimension.spawnItem(new ItemStack("minecraft:bone_meal", valueRemove))
                    }
                    writeDynamicData(dynamicName, { ...dynamicData, boneMeal_storage: dynamicData.boneMeal_storage - valueRemove })
                }
            })
    }

    if (!player.isSneaking) {
        interact_mainGUI()
    }
}

function autoHarvest_onTick(event) {
    const { block } = event;
    const harvestData = autoHarvestData[block.typeId];
    if (!harvestData) return; // Se o bloco não possuir dados de colheita, sai.

    const redstonePower = new RedstonePowerChecker(block);
    const isPowered = redstonePower.getMaxPower() > 0;

    // Atualiza o estado do componente conforme o sinal de redstone.
    block.setPermutation(block.permutation.withState("ext:auto_harvest_enable", isPowered));

    const dynamicName = dynamicFormat(block);
    // Atualiza dados dinâmicos com a configuração base se ainda não estiverem definidos
    const baseConfig = autoHarvest_config[block.typeId];
    updateDynamicData(dynamicName, baseConfig);
    let dynamicData = fetchDynamicData(dynamicName) || {};

    // Verifica o timer global para sincronização (ex.: a cada 20 ticks)
    const globalTimer = fetchDynamicData("timerGlobal");
    if (isPowered && globalTimer && globalTimer.timer === 20) {
        // Enquanto houver tempo e bonemeal disponível, decrementa o timer
        if (dynamicData.time > 0 && dynamicData.boneMeal_storage > 0) {
            writeDynamicData(dynamicName, { ...dynamicData, time: dynamicData.time - 1 });
        } else if (dynamicData.time <= 0) {
            squareGetBlocksBoneMeal({
                blockInput: block,
                range: harvestData.range,
                heightMin: harvestData.heightMin,
                heightMax: harvestData.heightMax,
                dynamicData: dynamicData
            })
            dynamicData.time = baseConfig.time;
            writeDynamicData(dynamicName, dynamicData);
        }
        squareGetBlocks({
            blockInput: block,
            range: harvestData.range,
            heightMin: harvestData.heightMin,
            heightMax: harvestData.heightMax,
            dynamicData: dynamicData
        });
    }
}

function teleportDroppedItems(dimension, location, targetItem, blockInput) {
    const teleportLocation = { ...blockInput.bottomCenter(), y: blockInput.bottomCenter().y + 1 };
    const items = dimension.getEntities({ location, maxDistance: 5, type: "minecraft:item" });

    items.forEach(entity => {
        const itemStack = entity.getComponent("minecraft:item").itemStack;
        if (targetItem.itemDropID.includes(itemStack.typeId)) {
            entity.teleport(teleportLocation);
        }
    });
}

function squareGetBlocksBoneMeal({ blockInput, range = 3, heightMin = 0, heightMax = 1, dynamicData }) {
    const { x: bx, y: by, z: bz } = blockInput.location;

    // Define os limites arredondados
    const minX = Math.round(bx - range);
    const maxX = Math.round(bx + range);
    const minY = Math.round(by - heightMin);
    const maxY = Math.round(by + heightMax);
    const minZ = Math.round(bz - range);
    const maxZ = Math.round(bz + range);

    const dimension = world.getDimension(blockInput.dimension.id);

    // Percorre cada bloco na área definida
    for (let x = minX; x <= maxX; x++) {
        for (let y = minY; y <= maxY; y++) {
            for (let z = minZ; z <= maxZ; z++) {
                const locationVector3 = { x, y, z };
                const blockInLoc = dimension.getBlock(locationVector3);
                if (!blockInLoc || blockInLoc.isAir) continue;

                // Verifica se o bloco está na lista de alvos para o efeito do bonemeal
                const targetItem = blocksCollectList.find(item => item?.typeId === blockInLoc.typeId);
                if (!targetItem || !targetItem.permutationData) continue;

                const { permutationName, permutationValue } = targetItem.permutationData;
                const currentValue = blockInLoc.permutation.getState(permutationName);

                // Se o bloco não estiver totalmente crescido, aplica o efeito do bonemeal
                if (currentValue < permutationValue) {
                    if (dynamicData.boneMeal_storage > 0) {
                        dimension.spawnParticle("minecraft:crop_growth_emitter", { ...blockInLoc.bottomCenter(), y: blockInLoc.bottomCenter().y + 0.5 })
                        blockInLoc.setPermutation(blockInLoc.permutation.withState(permutationName, currentValue + 1));
                        dynamicData.boneMeal_storage -= 0.25;
                        writeDynamicData(dynamicFormat(blockInput), dynamicData);
                    }
                }
            }
        }
    }
}

function squareGetBlocks({ blockInput, targetArrayList = blocksCollectList, range = 3, heightMin = 0, heightMax = 1 }) {
    const { x: bx, y: by, z: bz } = blockInput.location;

    // Define os limites arredondados
    const minX = Math.round(bx - range);
    const maxX = Math.round(bx + range);
    const minY = Math.round(by - heightMin);
    const maxY = Math.round(by + heightMax);
    const minZ = Math.round(bz - range);
    const maxZ = Math.round(bz + range);

    const dimension = world.getDimension(blockInput.dimension.id);

    // Itera pelos blocos dentro do volume definido
    for (let x = minX; x <= maxX; x++) {
        for (let y = minY; y <= maxY; y++) {
            for (let z = minZ; z <= maxZ; z++) {
                const locationVector3 = { x, y, z };
                const blockInLoc = dimension.getBlock(locationVector3);
                if (!blockInLoc || blockInLoc.isAir) continue;

                const targetItem = targetArrayList.find(item => item?.typeId === blockInLoc.typeId);
                if (!targetItem) continue;

                if (targetItem.permutationData) {
                    const { permutationName, permutationValue } = targetItem.permutationData;
                    const currentValue = blockInLoc.permutation.getState(permutationName);
                    const blockPermutationClone = blockInLoc.permutation.getAllStates();

                    if (currentValue !== permutationValue) continue;

                    dimension.runCommandAsync(`setblock ${x} ${y} ${z} air destroy`)
                        .then(() => {
                            teleportDroppedItems(dimension, locationVector3, targetItem, blockInput);
                            dimension.setBlockType(locationVector3, targetItem.typeId);
                            // Restaura os outros estados do bloco
                            Object.keys(blockPermutationClone).forEach(permutationKey => {
                                if (permutationKey !== permutationName) {
                                    const blockPlaced = dimension.getBlock(locationVector3);
                                    blockPlaced.setPermutation(blockPlaced.permutation.withState(permutationKey, blockPermutationClone[permutationKey]));
                                }
                            });
                        })
                        .catch(err => console.warn(err));

                } else if (targetItem.setblock) {
                    dimension.runCommandAsync(`setblock ${x} ${y} ${z} air destroy`)
                        .then(() => teleportDroppedItems(dimension, locationVector3, targetItem, blockInput))
                        .catch(err => console.error(err));
                }
            }
        }
    }
}

const autoHarvest_config = {
    "ext:auto_harvest_copper": {
        boneMeal_storage: 0,
        time: 60
    },
    "ext:auto_harvest_iron": {
        boneMeal_storage: 0,
        time: 80
    },
    "ext:auto_harvest_diamond": {
        boneMeal_storage: 0,
        time: 100
    },
};

const autoHarvestData = {
    "ext:auto_harvest_copper": {
        range: 5,
        heightMin: 1,
        heightMax: 1
    },
    "ext:auto_harvest_iron": {
        range: 9,
        heightMin: 2,
        heightMax: 2
    },
    "ext:auto_harvest_diamond": {
        range: 13,
        heightMin: 4,
        heightMax: 4
    }
};

const blocksCollectList = [
    {
        typeId: "minecraft:wheat",
        itemDropID: ["minecraft:wheat", "minecraft:wheat_seeds"],
        permutationData: {
            permutationName: "growth",
            permutationValue: 7
        }
    },
    {
        typeId: "minecraft:potatoes",
        itemDropID: ["minecraft:potato", "minecraft:poisonous_potato"],
        permutationData: {
            permutationName: "growth",
            permutationValue: 7
        }
    },
    {
        typeId: "minecraft:cocoa",
        itemDropID: ["minecraft:cocoa_beans"],
        permutationData: {
            permutationName: "age",
            permutationValue: 2
        }
    },
    {
        typeId: "minecraft:beetroot",
        itemDropID: ["minecraft:beetroot", "minecraft:beetroot_seeds"],
        permutationData: {
            permutationName: "growth",
            permutationValue: 7
        }
    },
    {
        typeId: "minecraft:carrots",
        itemDropID: ["minecraft:carrot"],
        permutationData: {
            permutationName: "growth",
            permutationValue: 7
        }
    },
    {
        typeId: "minecraft:sweet_berry_bush",
        itemDropID: ["minecraft:sweet_berries"],
        permutationData: {
            permutationName: "growth",
            permutationValue: 3
        }
    },
    {
        typeId: "minecraft:melon_block",
        itemDropID: ["minecraft:melon_slice"],
        setblock: "minecraft:air"
    },
    {
        typeId: "minecraft:pumpkin",
        itemDropID: ["minecraft:pumpkin"],
        setblock: "minecraft:air"
    }
];
