import { EnchantmentType, EntityComponentTypes, EquipmentSlot, ItemStack, system, world } from "@minecraft/server";
import { VFX_particle, fetchDynamicData, randomNumber, updateDynamicData, writeDynamicData, RedstonePowerChecker, isAllowedBlock } from "../extended_functions.js";

const drying_dynamicFormat = (location) => `drying_frame=${JSON.stringify(location)}`;

world.beforeEvents.worldInitialize.subscribe((event) => {
    event.blockComponentRegistry.registerCustomComponent("ext:drying_frame_component", {
        onTick: drying_frame_tick,
        onPlayerInteract: drying_frame_interact,
        onPlayerDestroy: drying_frame_destroy,
        beforeOnPlayerPlace: drying_frame_place
    });
});

function drying_frame_place(event) {
    const { dimension, block } = event;
    dimension.spawnEntity("ext:drying_frame_entity", { ...block.bottomCenter(), y: block.location.y });
}

function drying_frame_destroy(event) {
    const { dimension, block } = event;
    const entity = dimension.getEntitiesAtBlockLocation(block.bottomCenter())[0]
    if (!entity) return;
    const inventoryComponent = entity.getComponent("minecraft:inventory");
    if (inventoryComponent) {
        for (let slotIndex = 0; slotIndex < inventoryComponent.inventorySize; slotIndex++) {
            const item = inventoryComponent.container.getItem(slotIndex);
            if (item) {
                dimension.spawnItem(new ItemStack(item.typeId, item.amount), entity.location);
            }
        }
    }
}

// Função para calcular o valor final somando um RNG para cada item
function calculateFinalAmount(inputAmount, rawAmount) {
    if (typeof rawAmount === "string" && rawAmount.startsWith("RNG")) {
        const parts = rawAmount.split("=")[1].split(":");
        const min = parseInt(parts[0]);
        const max = parseInt(parts[1]);
        let total = 0;
        // Para cada item, soma um valor aleatório entre min e max
        for (let i = 0; i < inputAmount; i++) {
            total += randomNumber(min, max);
        }
        return Math.min(total, 64);
    } else {
        return inputAmount * Number(rawAmount);
    }
}

function drying_frame_tick(event) {
    const { block, dimension } = event;
    const dryingFrame_entity = dimension.getEntitiesAtBlockLocation(block.bottomCenter())[0];
    const drying_InventoryComponent = dryingFrame_entity?.getComponent(EntityComponentTypes.Inventory);
    const blockDirection = block.permutation.getState("minecraft:cardinal_direction")

    dryingFrame_entity?.setRotation({ x: 0, y: drying_frame_rotation[blockDirection] })

    if (!drying_InventoryComponent) return;
    const itemInDrying = drying_InventoryComponent.container.getItem(0);
    const dynamicName = world.getDynamicPropertyIds().find(name => name == drying_dynamicFormat(block.bottomCenter()));
    let dynamicTimer = fetchDynamicData(drying_dynamicFormat(block.bottomCenter()))?.timer;

    dryingFrame_entity.nameTag = (itemInDrying?.amount != undefined && dynamicTimer != undefined)
        ? `§6${dynamicTimer}`
        : ``;

    if (itemInDrying) {
        dryingFrame_entity.runCommandAsync(`replaceitem entity @s slot.weapon.mainhand 0 ${itemInDrying.typeId}`);
    } else {
        dryingFrame_entity.runCommandAsync(`replaceitem entity @s slot.weapon.mainhand 0 minecraft:air`);
    }

    if (world.getTimeOfDay() > 12800 && world.getTimeOfDay() < 25000) return;

    if (!dynamicName) return;

    if (itemInDrying) {
        if (dynamicTimer <= 0) {
            if (!drying_frame_itemsList[itemInDrying.typeId]) return;
            world.setDynamicProperty(dynamicName, undefined);
            const rawAmount = drying_frame_itemsList[itemInDrying.typeId].amount;
            const finalOutputAmount = calculateFinalAmount(itemInDrying.amount, rawAmount);
            dryingFrame_entity.runCommandAsync(`replaceitem entity @s slot.inventory 0 ${drying_frame_itemsList[itemInDrying.typeId].output} ${finalOutputAmount}`);
            return;
        }
        if (dynamicTimer > 0 && fetchDynamicData("timerGlobal").timer == 20) {
            writeDynamicData(drying_dynamicFormat(block.bottomCenter()), { ...fetchDynamicData(dynamicName), timer: dynamicTimer - 1 });
            return;
        }
    } else {
        world.setDynamicProperty(dynamicName, undefined);
    }
}

function drying_frame_interact(event) {
    const { block, player } = event;
    const dimension = world.getDimension(block.dimension.id);
    const dryingFrame_entity = dimension.getEntitiesAtBlockLocation(block.bottomCenter())[0];

    const player_InventoryComponent = player?.getComponent(EntityComponentTypes.Inventory);
    const drying_InventoryComponent = dryingFrame_entity?.getComponent(EntityComponentTypes.Inventory);

    if (!player_InventoryComponent && !drying_InventoryComponent) return;

    const itemInMainhand = player_InventoryComponent.container.getItem(player.selectedSlotIndex);
    const itemInDrying = drying_InventoryComponent.container.getItem(0);

    if (itemInDrying) {
        dryingFrame_entity.runCommandAsync(`replaceitem entity @s slot.inventory 0 minecraft:air`).then(() => {
            dimension.spawnItem(new ItemStack(itemInDrying.typeId, itemInDrying.amount), block.bottomCenter());
        });
    }
    if (drying_frame_itemsList[itemInMainhand?.typeId]) {
        dryingFrame_entity.runCommandAsync(`replaceitem entity @s slot.inventory 0 ${itemInMainhand?.typeId ?? "minecraft:air"} ${itemInMainhand?.amount ?? 0}`).then(() => {
            writeDynamicData(drying_dynamicFormat(block.bottomCenter()), { timer: (itemInMainhand.amount * drying_frame_itemsList[itemInMainhand.typeId].timer), blockData: block, dimension: block.dimension.id });
            player.runCommandAsync(`replaceitem entity @s slot.weapon.mainhand 0 minecraft:air`);
        });
    }
}

system.runInterval(() => {
    world.getAllPlayers().forEach(player => {
        const dimension = world.getDimension(player.dimension.id);
        dimension.getEntities({ type: "ext:drying_frame_entity" }).forEach(entity => {
            const blockInLocation = dimension.getBlock(entity.location)
            if (blockInLocation?.typeId != "ext:drying_frame_block" && blockInLocation?.typeId != undefined) {
                if (entity) {
                    const inventoryComponent = entity.getComponent("minecraft:inventory");
                    if (inventoryComponent) {
                        for (let slotIndex = 0; slotIndex < inventoryComponent.inventorySize; slotIndex++) {
                            const item = inventoryComponent.container.getItem(slotIndex);
                            if (item) {
                                dimension.spawnItem(new ItemStack(item.typeId, item.amount), entity.location);
                            }
                        }
                    }
                    entity.remove();
                }
            }
        })
    })
}, 1)

const drying_frame_itemsList = {
    "minecraft:rotten_flesh": { output: "ext:rotten_flesh_dry", timer: 2, amount: 1 },
    "minecraft:short_grass": { output: "minecraft:wheat_seeds", timer: 3, amount: "RNG=1:2" },
    "minecraft:kelp": { output: "minecraft:dried_kelp", timer: 0.5, amount: "RNG=1:2" },
};

const drying_frame_rotation = {
    "north": 0,
    "south": 0,
    "west": 90,
    "east": 90,
}