import { ItemStack, world } from "@minecraft/server";

world.beforeEvents.worldInitialize.subscribe((event) => {
    event.blockComponentRegistry.registerCustomComponent("ext:filter_component", {
        onTick: filterVerifier,
    });
});

const blockAllowList = ["minecraft:hopper", "minecraft:chest", "minecraft:barrel"];

const entityAllowList = ["minecraft:hopper_minecart", "minecraft:chest_minecart", "minecraft:chest_boat", "ext:reinforced_furnace_storage"];

const config = {
    teleportOffset: { x: 0.5, y: -0.3, z: 0.5 },
    hitboxOffsets: [{ y: 1 }, { y: 2 }],
    detectionDistance: 0.8,
    maxDistance: 0.8,
};

function filterVerifier(event) {
    const { block } = event;
    const dimension = world.getDimension(block.dimension.id);
    const blockCenter = block.bottomCenter();

    const hitboxPositions = config.hitboxOffsets.map((offset) => ({
        x: blockCenter.x,
        y: block.location.y + offset.y,
        z: blockCenter.z,
    }));

    const blockAbove = dimension.getBlock({
        x: block.location.x,
        y: block.location.y + 1,
        z: block.location.z,
    });

    // Container abaixo (destino de transferência)
    const blockBelow = dimension.getBlock({
        x: block.location.x,
        y: block.location.y - 1,
        z: block.location.z,
    });
    const containerBelowComponent = blockBelow ? blockBelow.getComponent("minecraft:inventory") : null;

    const surroundingOffsets = [
        { x: 1, y: 0, z: 0 },
        { x: -1, y: 0, z: 0 },
        { x: 0, y: 0, z: 1 },
        { x: 0, y: 0, z: -1 },
    ];
    const surroundingBlocks = surroundingOffsets.map((offset) =>
        dimension.getBlock({
            x: block.location.x + offset.x,
            y: block.location.y,
            z: block.location.z + offset.z,
        })
    );

    // Função auxiliar para extrair itens de um container (usado como filtro, se existir)
    const getFilterItems = (containerComponent) => {
        const filterList = [];
        for (let slot = 0; slot < containerComponent.container.size; slot++) {
            const item = containerComponent.container.getItem(slot);
            if (item) filterList.push({ id: item.typeId, amount: item.amount, slot });
        }
        return filterList;
    };

    // Procura por um container permitido nas laterais para definir o filtro
    let filterList = null;
    for (const currentBlock of surroundingBlocks) {
        if (currentBlock && blockAllowList.includes(currentBlock.typeId)) {
            const container = currentBlock.getComponent("minecraft:inventory");
            if (container) {
                filterList = getFilterItems(container);
                break;
            }
        }
    }
    // Se nenhum filtro for encontrado, processa todos os itens (comportamento vanilla)
    if (filterList === null) {
        filterList = [];
    }

    // Processa itens dropados na área do funil.
    // Se não houver container abaixo, não remove o item (funil vanilla não “sugar” se não houver destino).
    const handleDroppedItems = (filterList) => {
        const itemEntities = [];
        hitboxPositions.forEach((pos) => {
            const entities = dimension.getEntities({
                location: pos,
                type: "minecraft:item",
                maxDistance: config.detectionDistance,
            });
            entities.forEach((entity) => {
                if (entity.getComponent("minecraft:item")) itemEntities.push(entity);
            });
        });

        itemEntities.forEach((entity) => {
            const itemComp = entity.getComponent("minecraft:item");
            if (!itemComp) return;
            const itemStack = itemComp.itemStack;
            let shouldProcess = false;
            // Se não há filtro, processa todos os itens.
            if (filterList.length === 0) {
                shouldProcess = true;
            } else {
                for (const filterItem of filterList) {
                    if (itemStack.typeId === filterItem.id) {
                        shouldProcess = true;
                        break;
                    }
                }
            }
            if (shouldProcess && containerBelowComponent) {
                const newStack = new ItemStack(itemStack.typeId, itemStack.amount);
                const remainder = containerBelowComponent.container.addItem(newStack);
                // Remove o item somente se transferido por completo
                if (!remainder || remainder.amount === 0) {
                    entity.remove();
                }
            }
        });
    };

    // Transfere 1 unidade do inventário de origem para o container abaixo.
    // Se não houver container abaixo, não efetua a transferência.
    const spawnItemFromSource = (sourceInv, itemData) => {
        const { slot, id } = itemData;
        const newStack = new ItemStack(id, 1);
        if (containerBelowComponent) {
            const remainder = containerBelowComponent.container.addItem(newStack);
            // Se houver resto, não transfere o item
            if (remainder && remainder.amount > 0) return;
            const currentItem = sourceInv.container.getItem(slot);
            if (currentItem && currentItem.amount > 1) {
                sourceInv.container.setItem(slot, new ItemStack(id, currentItem.amount - 1));
            } else {
                sourceInv.container.setItem(slot, undefined);
            }
        }
    };

    // Transfere itens do inventário de origem; se não houver filtro, transfere todos.
    const handleContainerTransfer = (sourceInventoryComponent, filterList) => {
        if (!sourceInventoryComponent) return;
        const sourceItems = [];
        for (let slot = 0; slot < sourceInventoryComponent.container.size; slot++) {
            const item = sourceInventoryComponent.container.getItem(slot);
            if (item) sourceItems.push({ id: item.typeId, amount: item.amount, slot });
        }
        let itemTransferred = false;
        for (const sourceItem of sourceItems) {
            let shouldProcess = false;
            if (filterList.length === 0) {
                shouldProcess = true;
            } else {
                for (const filterItem of filterList) {
                    if (sourceItem.id === filterItem.id) {
                        shouldProcess = true;
                        break;
                    }
                }
            }
            if (shouldProcess && !itemTransferred) {
                spawnItemFromSource(sourceInventoryComponent, sourceItem);
                itemTransferred = true;
            }
        }
    };

    // Processa entidades permitidas próximas ao funil.
    for (const entityType of entityAllowList) {
        const nearbyEntities = dimension.getEntities({
            location: { ...block.location, y: block.location.y + 1 },
            type: entityType,
            maxDistance: config.maxDistance,
        });

        for (const entity of nearbyEntities) {
            const entityInventory = entity.getComponent("minecraft:inventory");
            if (!entityInventory) continue;

            // Processa itens dropados e transferência do inventário
            handleDroppedItems(filterList);
            handleContainerTransfer(entityInventory, filterList);

            const blockAboveEntity = dimension.getBlock({
                x: entity.location.x,
                y: Math.ceil(entity.location.y + 1),
                z: entity.location.z,
            });
            if (blockAboveEntity) {
                const topInventory = blockAboveEntity.getComponent("minecraft:inventory");
                if (topInventory) handleContainerTransfer(topInventory, filterList);
            }
        }
    }

    // Fallback: se não houver entidades, processa itens dos blocos vizinhos.
    handleDroppedItems(filterList);
    if (blockAbove) {
        const topInventory = blockAbove.getComponent("minecraft:inventory");
        if (topInventory) handleContainerTransfer(topInventory, filterList);
    }
}