import { EnchantmentType, system, world } from "@minecraft/server";
import { ModalFormData } from "@minecraft/server-ui";
import { VFX_particle, fetchDynamicData, randomNumber, updateDynamicData, writeDynamicData, RedstonePowerChecker, isAllowedBlock } from "../extended_functions.js";

const dynamicFormat = (block) => `xp_storage=${JSON.stringify(block.location)}`;
const dynamicFormatSettings = (block) => `xp_storage_settings=${JSON.stringify(block.location)}`;

world.beforeEvents.worldInitialize.subscribe((event) => {
    event.blockComponentRegistry.registerCustomComponent("ext:xp_storage_component", {
        onTick: xp_storage_tick,
        beforeOnPlayerPlace: xp_storage_place,
        onPlayerInteract: xp_storage_interact
    });
});

world.beforeEvents.playerBreakBlock.subscribe((event) => {
    const { player, block } = event
    if (block.typeId == "ext:xp_storage") {
        const dynamicSettings = fetchDynamicData(dynamicFormatSettings(block))
        if (dynamicSettings.private == true && dynamicSettings.ownerName != player.name) {
            event.cancel = true;
        }
    }
})

function xp_storage_interact(event) {
    const { block, player, dimension } = event;
    const dynamicData = fetchDynamicData(dynamicFormat(block))
    const dynamicSettings = fetchDynamicData(dynamicData.dynamicSettingsName)

    if (dynamicSettings.private == true && dynamicSettings.ownerName != player.name) return;

    if (player.isSneaking) {
        const translateText = {
            rawtext: [
                { text: "§e" }, { translate: "tile.ext:xp_storage_gui.text.1" }, { text: ` §a${dynamicData.levelInBlock}` },
                { text: "\n§e" }, { translate: "tile.ext:xp_storage_gui.text.2" }, { text: ` §a${dynamicData.addLevelTime}` },
            ]
        }
        player.sendMessage(translateText)
        return;
    }
    const toggle1_translate = {
        rawtext: [
            { text: "\n" },
            { text: "§e" }, { translate: "tile.ext:xp_storage_gui.text.1" }, { text: ` §a${dynamicData.levelInBlock}` },
            { text: "\n§e" }, { translate: "tile.ext:xp_storage_gui.text.2" }, { text: ` §a${dynamicData.addLevelTime}` },
            { text: "\n\n§r" },
            { translate: "tile.ext:xp_storage_gui.text.3" },
        ]
    }
    new ModalFormData()
        .title({ translate: "tile.ext:xp_storage.name" })
        .toggle(toggle1_translate, dynamicSettings.private)
        .show(player).then(response => {
            const { formValues, canceled } = response
            if (!canceled) {
                if (formValues[0] == dynamicSettings.private) return;
                if (formValues[0]) {
                    player.sendMessage({ translate: "tile.ext:xp_storage_gui.text.4" })
                } else {
                    player.sendMessage({ translate: "tile.ext:xp_storage_gui.text.5" })
                }
                writeDynamicData(dynamicData.dynamicSettingsName, { ...dynamicSettings, private: formValues[0] })
            }
        })
}

function xp_storage_place(event) {
    const { block, player } = event;
    updateDynamicData(dynamicFormat(block), { levelInBlock: 0, dimensionID: block.dimension.id, addLevelTime: 0, dynamicSettingsName: dynamicFormatSettings(block) });
    updateDynamicData(dynamicFormatSettings(block), { ownerName: player.name, private: false });
}

function xp_storage_tick(event) {
    try {
        const { block, dimension } = event;
        const playersInBlock = dimension.getEntitiesAtBlockLocation({ ...block.location, y: block.location.y + 1 });
        const dynamicNameData = dynamicFormat(block);
        const dynamicData = fetchDynamicData(dynamicNameData);
        if (dynamicData == undefined) dimension.runCommandAsync(`setblock ${block.location.x} ${block.location.y} ${block.location.z} air destroy`);
        const dynamicSettings = fetchDynamicData(dynamicData.dynamicSettingsName);

        playersInBlock.forEach(player => {
            if (dynamicSettings.private == true && dynamicSettings.ownerName != player.name) return;
            const player_levelCurrent = player.level;
            if (player.isSneaking) {
                if (player_levelCurrent > 0) {
                    player.addLevels(-1)
                    writeDynamicData(dynamicNameData, { ...dynamicData, levelInBlock: dynamicData.levelInBlock + 1, addLevelTime: Math.round(dynamicData.levelInBlock * 16) })
                }
            } else {
                if (dynamicData.levelInBlock > 0) {
                    player.addLevels(1)
                    writeDynamicData(dynamicNameData, { ...dynamicData, levelInBlock: dynamicData.levelInBlock - 1, addLevelTime: Math.round(dynamicData.levelInBlock * 16) })
                }
            }
        });
    } catch { }
}

system.runInterval(() => {
    try {
        world.getDynamicPropertyIds().forEach(dynamicName => {
            if (dynamicName.startsWith("xp_storage=")) {
                const dynamicData = fetchDynamicData(dynamicName)
                const locationFormat = JSON.parse(dynamicName.split("=")[1])
                const blockInLocation = world.getDimension(dynamicData.dimensionID).getBlock(locationFormat)
                if (blockInLocation.typeId != "ext:xp_storage") {
                    world.setDynamicProperty(dynamicName, undefined)
                    world.setDynamicProperty(dynamicData.dynamicSettingsName, undefined)
                } else {
                    blockInLocation.setPermutation(blockInLocation.permutation.withState("ext:activate_animation", dynamicData.levelInBlock > 0));
                    if (dynamicData.addLevelTime > 0 && fetchDynamicData("timerGlobal").timer == 20) {
                        writeDynamicData(dynamicName, { ...dynamicData, addLevelTime: dynamicData.addLevelTime - 1 })
                    } else if (dynamicData.addLevelTime <= 0) {
                        writeDynamicData(dynamicName, { ...dynamicData, levelInBlock: Math.round(dynamicData.levelInBlock * 1.125), addLevelTime: Math.round(dynamicData.levelInBlock * 16) })
                    }
                }
            }
        })
    } catch { }
})