import { world, system } from "@minecraft/server";

export class AnimationBlock {
    constructor(block, animationFrames = [], animationInterval = 0, permutationID = "ext:animation_frame", loop = false) {
        this.block = block;
        this.animationFrames = animationFrames;
        this.permutationID = permutationID;
        this.animationInterval = animationInterval;
        this.frameIndex = 0;
        this.loop = loop;
        this.isRunning = false;
        this.currentFrame = null;
    }

    start() {
        if (this.isRunning) return;
        this.isRunning = true;

        const animate = () => {
            if (!this.isRunning) return;

            if (this.frameIndex < this.animationFrames.length) {
                // Atualiza o bloco primeiro
                this.block?.setPermutation(
                    this.block?.permutation.withState(this.permutationID, this.animationFrames[this.frameIndex])
                );

                // Depois atualiza o currentFrame e faz o log
                this.currentFrame = this.animationFrames[this.frameIndex];

                this.frameIndex++;
            } else if (this.loop) {
                this.frameIndex = 0;

                // Atualiza o bloco primeiro
                this.block?.setPermutation(
                    this.block?.permutation.withState(this.permutationID, this.animationFrames[this.frameIndex])
                );

                // Depois atualiza o currentFrame e faz o log
                this.currentFrame = this.animationFrames[this.frameIndex];
            }

            if (this.frameIndex < this.animationFrames.length || this.loop) {
                system.runTimeout(animate, this.animationInterval);

            } else {
                this.isRunning = false;
            }
        };

        animate();
    }

    stop() {
        this.isRunning = false;
        this.currentFrame = null;
    }

    animationIsRunning() {
        return this.isRunning;
    }

    getCurrentFrame() {
        return this.currentFrame;
    }
}

export class RedstonePowerChecker {
    constructor(blockInput) {
        this.blockInput = blockInput;
        this.directions = [
            { x: blockInput.location.x, y: blockInput.location.y, z: blockInput.location.z },
            { x: blockInput.location.x + 1, y: blockInput.location.y, z: blockInput.location.z },
            { x: blockInput.location.x - 1, y: blockInput.location.y, z: blockInput.location.z },
            { x: blockInput.location.x, y: blockInput.location.y + 1, z: blockInput.location.z },
            { x: blockInput.location.x, y: blockInput.location.y - 1, z: blockInput.location.z },
            { x: blockInput.location.x, y: blockInput.location.y, z: blockInput.location.z + 1 },
            { x: blockInput.location.x, y: blockInput.location.y, z: blockInput.location.z - 1 },
        ];
    }

    getMaxPower() {
        let maxPower = 0;

        for (const dir of this.directions) {
            const neighborBlock = this.getNeighborBlock(dir);
            if (neighborBlock) {
                const power = neighborBlock.getRedstonePower();
                if (power !== undefined) {
                    maxPower = Math.max(maxPower, power);
                }
            }
        }

        return maxPower;
    }

    getNeighborBlock(direction) {
        return world.getDimension(this.blockInput.dimension.id).getBlock(direction);
    }
}

export function VFX_particle(vector3, dimensionEntityOrBlock, particleID, size, maxParticles = 100, yFIX = 0) {

    const particles = Math.min(maxParticles, Math.max(1, Math.floor(360 / size))); // Garante pelo menos 1 partícula
    const dimension = world.getDimension(dimensionEntityOrBlock.dimension.id);

    for (let i = 0; i < particles; i++) {
        const theta = Math.random() * Math.PI * 2;
        const phi = Math.acos(1 - 2 * Math.random()); // Distribuição uniforme na esfera

        const randomRadius = size * (0.75 + Math.random() * 0.5); // Evita valores negativos

        const xr = vector3.x + 0.5 + randomRadius * Math.sin(phi) * Math.cos(theta);
        const zr = vector3.z + 0.5 + randomRadius * Math.sin(phi) * Math.sin(theta);
        const yr = Math.max(-64, Math.min(vector3.y + randomRadius * Math.cos(phi) + yFIX, 320));

        dimension.spawnParticle(particleID, { x: xr, y: yr, z: zr });
    }
}

export function randomNumber(min, max, round = true) {
    if (min > max) {
        throw new Error("O valor mínimo não pode ser maior que o valor máximo.");
    }
    function generateSeed() {
        const now = Date.now();
        const randomPart = Math.random();
        return (now * randomPart) % 1e8;
    }
    const a = 1664525;
    const c = 1013904223;
    const m = Math.pow(2, 32);
    function lcg(seed) {
        return function () {
            seed = (a * seed + c) % m;
            return seed / m;
        }
    }
    const seed = generateSeed();
    const generator = lcg(seed);
    const randomValue = generator() * (max - min) + min;
    const boundedValue = Math.max(min, Math.min(randomValue, max));
    return round ? Math.round(boundedValue) : boundedValue;
}

export const findIndexPerString = (list, input = "test") => list.findIndex(obj => { return input.toLocaleLowerCase() == obj.name.toLocaleLowerCase() })

export function fetchDynamicData(name) {
    try {
        const dynamicProperty = world.getDynamicProperty(name);
        if (dynamicProperty) {
            try {
                return JSON.parse(dynamicProperty);
            } catch (jsonError) {
                return undefined
            }
        } else {
            return undefined
        }
    } catch (error) {
        console.error(`Erro ao buscar propriedade dinâmica '${name}':`, error);
    }
}

export function writeDynamicData(name, input) {
    try {
        if (input == undefined || input == null) {
            world.setDynamicProperty(name, undefined);
            return;
        }
        const inputFormat = JSON.stringify(input);
        world.setDynamicProperty(name, inputFormat);
        return true;
    } catch (error) {
        console.error(`Erro ao escrever propriedade dinâmica '${name}':`, error);
        return false;
    }
}

export function updateDynamicData(dynamicName, structureConfig = null) {
    try {

        const dynamicData = fetchDynamicData(dynamicName);

        if (dynamicData == null || dynamicData == undefined) {
            writeDynamicData(dynamicName, structureConfig);
        }

        return dynamicData

    } catch (error) {
        writeDynamicData(dynamicName, structureConfig);
        return fetchDynamicData(dynamicName);
    }
}

export function isAllowedBlock(block, suffixList = allowedSuffixes_list, tagList = allowedTags_list) {
    if (!block) return false;

    // Se o bloco possui a propriedade "flower_type", é permitido
    if (block?.permutation?.getAllStates?.()["flower_type"]) {
        return true;
    }

    // Se o bloco possui alguma das tags permitidas
    if (tagList.some(tag => block.hasTag(tag))) {
        return true;
    }

    // Verifica se o block.typeId termina com algum dos sufixos permitidos
    const typeId = block.typeId;
    for (const suffix of suffixList) {
        if (typeId.endsWith(suffix)) {
            return true;
        }
    }

    return false;
}

export function hasDynamicData(name) {
    try {
        const dynamicList = world.getDynamicPropertyIds();
        if (dynamicList.includes(name)) {
            return true;
        } else {
            return false;
        }
    } catch (error) {
        console.error(`Erro ao verificar existência da propriedade dinâmica '${name}':`, error);
        return false;
    }
}

export function spawnEntityInRegion(entityID, loc, rad = 4, height = 0, minDistance = 5) {
    let attempts = 0;
    const maxAttempts = 256;

    while (attempts < maxAttempts) {
        const xRange = [loc.x - rad, loc.x + rad];
        const yRange = [loc.y - height, loc.y + height];
        const zRange = [loc.z - rad, loc.z + rad];

        let spX = randomNumber(Math.min(...xRange), Math.max(...xRange));
        let spY = randomNumber(Math.min(...yRange), Math.max(...yRange));
        let spZ = randomNumber(Math.min(...zRange), Math.max(...zRange));

        const distanceX = Math.abs(spX - loc.x);
        const distanceZ = Math.abs(spZ - loc.z);

        if (distanceX < minDistance || distanceZ < minDistance) {
            attempts++;
            continue;
        }

        const Dimension = getDimensionEXT(targetSelected);

        const block0 = Dimension.getBlock({ x: spX, y: spY + 1, z: spZ });
        const block1 = Dimension.getBlock({ x: spX, y: spY, z: spZ });
        const block2 = Dimension.getBlock({ x: spX, y: spY - 1, z: spZ });

        const spawnCondition = block0.typeId === "minecraft:air" && block1.typeId === "minecraft:air" && block2.typeId !== "minecraft:air";

        if (spawnCondition) {
            let entitySpawned = Dimension.spawnEntity(entityID, { x: spX, y: spY, z: spZ });
            return entitySpawned;
        }

        attempts++;
    }

    console.warn("Falha ao gerar entidade após várias tentativas.");
}

export class BlockManager {

    replaceBlock(vector3, blockInLoc, replacementBlock, keepPermutation) {
        if (keepPermutation) {
            const permutationData = blockInLoc.permutation.getState("torch_facing_direction");
            dimension.setBlockType(vector3, replacementBlock);
            dimension.setBlockPermutation(
                vector3,
                blockInLoc.permutation.withState("torch_facing_direction", permutationData)
            );
        } else {
            dimension.setBlockType(vector3, replacementBlock);
        }
    }

    squareGetBlocks({
        locationStart,
        allSides = false,
        blockTargetID = "none",
        replacementBlock = "minecraft:redstone_torch",
        keepPermutation = true,
        rad = 20,
        height = 4
    } = {}) {
        try {
            const { x: originX, y: originY, z: originZ } = locationStart;

            const [minX, minY, minZ] = [
                Math.round(originX - rad),
                Math.round(allSides ? originY - height : originY),
                Math.round(originZ - rad)
            ];
            const [maxX, maxY, maxZ] = [
                Math.round(originX + rad),
                Math.round(originY + height),
                Math.round(originZ + rad)
            ];

            for (let x = minX; x <= maxX; x++) {
                for (let y = minY; y <= maxY; y++) {
                    for (let z = minZ; z <= maxZ; z++) {
                        const vector3 = { x: x, y: y, z: z };
                        const blockInLoc = dimension.getBlock(vector3);

                        if (blockTargetID !== "none" && blockInLoc.typeId === blockTargetID && blockInLoc.typeId !== "minecraft:air") {
                            this.replaceBlock(vector3, blockInLoc, replacementBlock, keepPermutation);
                        }
                    }
                }
            }
        } catch { }
    }
}

export const allowedSuffixes_list = [
    "_slab",
    "_double_slab",
    "_fence_gate",
    "_carpet",
    "_pressure_plate",
    "_button",
    "_wire",
    "_repeater",
    "_comparator",
    "_banner",
    "_layer",
    "_sign",
    "_leaves",
    "_rod",
    "minecraft:lever",
    "minecraft:torch",
    "minecraft:bell",
    "minecraft:tripwire_hook",
    "minecraft:rose_bush",
    "minecraft:lilac",
    "minecraft:peony",
    "minecraft:sunflower",
    "minecraft:heavy_core",
    "minecraft:web",
    "minecraft:bed",
    "minecraft:daylight_detector",
    "minecraft:deadbush",
    "minecraft:pointed_dripstone",
    "minecraft:daylight_detector_inverted",
    "minecraft:short_grass",
    "minecraft:tall_grass",
    "minecraft:bamboo",
    "minecraft:fern",
    "minecraft:cocoa",
    "minecraft:vine",
    "minecraft:dandelion",
    "minecraft:fire",
];

export const allowedTags_list = [
    "minecraft:not_feature_replaceable",
    "minecraft:plant",
    "minecraft:rail",
    "minecraft:trapdoors",
    "minecraft:snow",
    "minecraft:text_sing",
];