import { ItemStack, world } from "@minecraft/server";
import { fetchDynamicData, updateDynamicData, writeDynamicData, VFX_particle, randomNumber } from "../extended_functions.js";

world.beforeEvents.worldInitialize.subscribe(event => {
  event.itemComponentRegistry.registerCustomComponent("ext:rotten_flesh_powder_component", {
    onUseOn: rottenFleshPowderUse,
  });
});

function weightedRandom(options) {
  if (!options || options.length === 0) return undefined;
  const totalWeight = options.reduce((sum, option) => sum + option.weight, 0);
  let randomWeight = Math.random() * totalWeight;
  for (const option of options) {
    if (option.weight < 0) continue;
    if (randomWeight < option.weight) return option.type;
    randomWeight -= option.weight;
  }
  return options[options.length - 1].type;
}

class GrowthUtils {
  static getGrowthInfo(block) {
    const possibleStates = ["growth", "age", "ext:growth"];
    for (const state of possibleStates) {
      const stateValue = block.permutation.getState(state);
      if (stateValue !== undefined) {
        const parsedValue = Number(stateValue);
        if (!isNaN(parsedValue)) return { permutation: state, value: parsedValue };
      }
    }
    return null;
  }

  static applyGrowth(block, stateKey, increment, maxGrowth) {
    const currentValue = Number(block.permutation.getState(stateKey));
    const newGrowthValue = Math.min(currentValue + increment, maxGrowth);
    try {
      const newPermutation = block.permutation.withState(stateKey, newGrowthValue);
      block.setPermutation(newPermutation);
    } catch (error) {
      console.error(`Erro ao definir permutação para ${block.typeId}:`, error);
    }
  }
}

class CustomBoneMeal {
  constructor(defaultGrowthValue = 7) {
    this.defaultGrowthValue = defaultGrowthValue;
  }

  fertilizeOne_growth(event, growthListValue = [7], particleID = "minecraft:crop_growth_emitter", soundID = "item.bone_meal.use") {
    const { source, block, itemStack } = event;
    const dimension = world.getDimension(block.dimension.id);
    const inventoryComponent = source.getComponent("minecraft:inventory");
    const growthInfo = GrowthUtils.getGrowthInfo(block);
    if (growthInfo === null) return;
    const exception = growthListException.find(ex => ex.typeId === block.typeId);
    let growthIncrement = growthListValue[randomNumber(0, growthListValue.length - 1)];
    let stateKey = growthInfo.permutation;
    let maxGrowth = this.defaultGrowthValue;
    if (exception) {
      growthIncrement = exception.growthList[randomNumber(0, exception.growthList.length - 1)];
      stateKey = exception.permutation;
      maxGrowth = exception.growthMax;
    }
    GrowthUtils.applyGrowth(block, stateKey, growthIncrement, maxGrowth);
    this._spawnEffects(dimension, block.bottomCenter(), particleID, soundID, source);
    this.consumeItem(source, itemStack, inventoryComponent);
  }

  fertilizeInArea(event, radius = 3, offsetY = 1, particleID = "minecraft:crop_growth_area_emitter", soundID = "item.bone_meal.use") {
    const { source, block, itemStack } = event;
    const dimension = world.getDimension(block.dimension.id);
    if (block.typeId !== "minecraft:grass_block" || !block.hasTag("fertilize_area")) return;
    const baseLocation = block.location;
    let fertilizedCount = 0;
    const defaultGrowthIncrement = 1;
    for (let dx = -radius; dx <= radius; dx++) {
      for (let dz = -radius; dz <= radius; dz++) {
        const distance = Math.sqrt(dx * dx + dz * dz);
        if (distance > radius) continue;
        if (Math.random() < distance / radius) continue;
        for (let dyOffset = 0; dyOffset <= 1; dyOffset++) {
          const targetPos = {
            x: Math.floor(baseLocation.x + dx),
            y: Math.floor(baseLocation.y + offsetY + dyOffset),
            z: Math.floor(baseLocation.z + dz)
          };
          const targetBlock = dimension.getBlock(targetPos);
          if (!targetBlock) continue;
          const blockBelow = dimension.getBlock({ x: targetPos.x, y: targetPos.y - 1, z: targetPos.z });
          if (!blockBelow) continue;
          if (targetBlock.isAir && blockBelow.typeId === "minecraft:grass_block") {
            const newType = weightedRandom(decorativeBlocks);
            if (newType) targetBlock.setType(newType);
            fertilizedCount++;
            continue;
          }
          const growthInfo = GrowthUtils.getGrowthInfo(targetBlock);
          if (growthInfo !== null) {
            let localGrowthIncrement = defaultGrowthIncrement;
            let stateKey = growthInfo.permutation;
            let maxGrowth = this.defaultGrowthValue;
            const exception = growthListException.find(ex => ex.typeId === targetBlock.typeId);
            if (exception) {
              localGrowthIncrement = exception.growthList[randomNumber(0, exception.growthList.length - 1)];
              stateKey = exception.permutation;
              maxGrowth = exception.growthMax;
            }
            GrowthUtils.applyGrowth(targetBlock, stateKey, localGrowthIncrement, maxGrowth);
            fertilizedCount++;
          }
        }
      }
    }
    if (fertilizedCount > 0) {
      dimension.spawnParticle(particleID, { ...block.location, y: block.location.y + 1 });
      source.playSound(soundID, { location: block.location });
      this.consumeItem(source, itemStack, source.getComponent("minecraft:inventory"));
    }
  }

  consumeItem(source, itemStack, inventoryComponent) {
    if (source.getGameMode() === "creative") return;
    const newAmount = itemStack.amount - 1;
    inventoryComponent.container.setItem(
      source.selectedSlot,
      newAmount > 0 ? new ItemStack(itemStack.typeId, newAmount) : null
    );
  }

  _spawnEffects(dimension, effectPos, particleID, soundID, source) {
    dimension.spawnParticle(particleID, effectPos);
    source.playSound(soundID, { location: effectPos });
  }
}

function rottenFleshPowderUse(event) {
  const { source, block } = event;
  const dimension = world.getDimension(block.dimension.id);
  const boneMealManager = new CustomBoneMeal();
  for (const transformData of blocksTransform) {
    if (transformData.type === block.typeId) {
      if (transformData.mode === "transform") {
        dimension.setBlockType(block.location, transformData.type_transform);
      } else if (transformData.mode === "tower") {
        let currentTowerHeight = 0;
        let currentY = block.location.y;
        while (true) {
          const pos = { x: block.location.x, y: currentY, z: block.location.z };
          const currentBlock = dimension.getBlock(pos);
          if (currentBlock && currentBlock.typeId === block.typeId) {
            currentTowerHeight++;
            currentY++;
          } else {
            break;
          }
        }
        let blocksToAdd = transformData.heightAdd;
        if (transformData.heightLimit > 0) {
          blocksToAdd = Math.min(transformData.heightAdd, transformData.heightLimit - currentTowerHeight);
        }
        for (let i = 0; i <= blocksToAdd; i++) {
          const targetPos = {
            x: block.location.x,
            y: block.location.y + currentTowerHeight + i,
            z: block.location.z
          };
          const targetBlock = dimension.getBlock(targetPos);
          if (targetBlock && targetBlock.isAir) {
            dimension.setBlockType(targetPos, block.typeId);
          }
        }
      }
      dimension.spawnParticle("minecraft:crop_growth_emitter", {
        ...block.bottomCenter(),
        y: block.bottomCenter().y + 0.5
      });
      break;
    }
  }
  if (event.block.hasTag("fertilize_area")) {
    boneMealManager.fertilizeInArea(event, 3, 1);
  } else {
    boneMealManager.fertilizeOne_growth(event);
  }
}

const growthListException = [
  {
    typeId: "minecraft:cocoa",
    permutation: "age",
    growthList: [1],
    growthMax: 2
  }
];

const blocksTransform = [
  {
    type: "minecraft:short_grass",
    mode: "transform",
    type_transform: "minecraft:tall_grass"
  },
  {
    type: "minecraft:fern",
    mode: "transform",
    type_transform: "minecraft:large_fern"
  },
  {
    type: "minecraft:reeds",
    mode: "tower",
    heightLimit: 2,
    heightAdd: 2
  },
  {
    type: "minecraft:bamboo",
    permutation: { name: "bamboo_stalk_thickness", value: "thick" },
    mode: "tower",
    heightLimit: 12,
    heightAdd: 1
  }
];

const decorativeBlocks = [
  { type: "minecraft:short_grass", weight: 60 },
  { type: "minecraft:fern", weight: 30 },
  { type: "minecraft:cornflower", weight: 10 },
  { type: "minecraft:dandelion", weight: 5 },
  { type: "minecraft:azure_bluet", weight: 5 }
];